import { Citation } from "./citation";
import { Member } from "./member";

export interface AuthorWeight {
    weight: number;
    author: Member
}

export interface CitationWeight {
    weight: number;
    citation: Citation;
}

export class Post {

    private static DEFAULT_WEIGHT = 1000;

    private title: string;
    private content: string;
    private authors: AuthorWeight[];
    private citations: CitationWeight[];
    private authorsWeightTotal: number;
    private citationsWeightTotal: number;

    constructor (title: string, content: string, authors: Member | Member[]) {
        this.title = title;
        this.content = content;
        this.authors = [];
        this.citations = [];
        this.authorsWeightTotal = 0;
        this.citationsWeightTotal = 0;
        if (!Array.isArray(authors)) authors = [authors];
        for (const author of <Member[]> authors) {
            this.authorsWeightTotal += Post.DEFAULT_WEIGHT;
            this.authors.push({
                weight: Post.DEFAULT_WEIGHT,
                author: author
            });
        }
    }

    public static serialize(post: Post, pretty: boolean = true) {
        //TODO: Need to convert citations and authors to IDs.
        if (!pretty) return JSON.stringify(Post);
        return JSON.stringify(Post, undefined, 2);
    }

    public static parse(serializedPost: string) {
        //TODO: Need to retrieve citations and authors from IDs.
        try {
            const parsed = JSON.parse(serializedPost);
            const title = <string> parsed?.title;
            const content = <string> parsed.content;
            const authors = <AuthorWeight[]> parsed.authors;
            const citations = <CitationWeight[]> parsed.citations;
            const post = new Post(title, content, authors[0].author);
            for (const author of authors) post.setAuthor(author.author, author.weight);
            for (const citation of citations) post.setCitation(citation.citation, citation.weight);
            return post;
        } catch(err) {
            throw new Error("Failed to parse the serialized post. \n  - " + err.message);
        }
    }

    public getTitle() { return this.title; }
    public getContent() { return this.content; }
    public getAuthors() { return this.authors.map(({author}) => author); }
    public getCitations() { return this.citations.map(({citation}) => citation); }
    public getAuthorsWeightTotal() { return this.authorsWeightTotal; }
    public getCitationWeightTotal() { return this.citationsWeightTotal; }

    public setAuthor(author: Member, weight: number = Post.DEFAULT_WEIGHT) {
        this.removeAuthor(author);
        this.authors.push({author, weight});
        this.authorsWeightTotal += weight;
    }

    public removeAuthor(author: Member) {
        const index = this.getAuthors().indexOf(author);
        if (index != -1) {
            const current = this.authors[index];
            this.authorsWeightTotal -= current.weight;
            this.authors.splice(index, 1);
        }
    }

    public setCitation(citation: Citation, weight: number = Post.DEFAULT_WEIGHT) {
        this.removeCitation(citation);
        this.citations.push({citation, weight});
        this.citationsWeightTotal += weight;
    }


    public removeCitation(citation: Citation) {
        const index = this.getCitations().indexOf(citation);
        if (index != -1) {
            const current = this.citations[index];
            this.citationsWeightTotal -= current.weight;
            this.citations.splice(index, 1);
        }
    }

    public getWeight(weighted: Member | Citation) {
        const forAuthor = weighted instanceof Member;
        const arr = forAuthor ? this.authors  : this.citations;
        const key = forAuthor ? "author" : "citation";
        const index = arr.findIndex(item => item[key] == weighted);
        if (index == -1) return -1;
        return arr[index].weight;
    }

}